import 'package:osam/domain/event/event.dart';
import 'package:osam/domain/state/base_state.dart';
import 'package:osam/domain/store/store.dart';
import 'package:osam/navigation/domain/navigation_state.dart';
import 'package:osam/presentation/presenter.dart';
import 'package:osam/util/state_stream.dart';

import 'navigation_presenter_interface.dart';

//todo: create event to catch in middleware
abstract class PresenterWithNavigation<ST extends BaseState<ST>, T> extends Presenter<Store<ST>>
    implements INavigation<T> {
  StateWithNavigation<T> get state;

  @override
  void pop() => store.dispatchEvent(event: Event(reducer: (_) => state..pop()));

  @override
  void push(T route) => store.dispatchEvent(event: Event(reducer: (_) => state..push(route)));

  @override
  void pushReplacement(T route) =>
      store.dispatchEvent(event: Event(reducer: (_) => state..pushReplacement(route)));

  @override
  StateStream<List<T>> get routeStream => state.propertyStream((state) => state.routes);
}

abstract class PresenterWithTabNavigation<ST extends BaseState<ST>, T> extends Presenter<Store<ST>>
    with ITabNavigation<T> {
  TabNavigationState<T> get state;

  @override
  StateStream<T> get tabStream =>
      StateStream((state.propertyStream((state) => state.currentTab)), state.currentTab);

  @override
  void changeTab(T tab) =>
      store.dispatchEvent(event: Event(reducer: (_) => state..currentTab = tab));
}
